@@ -71,7 +71,6 @@
 #include <pwd.h>
 #include <errno.h>
 
-
 // if you want to read good code skip this hacked up file!
 
 typedef struct {
@@ -82,23 +81,23 @@ typedef struct {
 	Encoding	*encoding;
 } XFSFont;
 
-XFSFont xfsFont[MAXOPENFONTS];
+static XFSFont xfsFont[MAXOPENFONTS];
 
 #ifndef MAGNIFY
 int MAGNIFY = 0;
 #endif /* MAGNIFY */
 
-uint16_t maxLastChar = 255;
+static uint16_t maxLastChar = 255;
 
 static bool ttdb_needs_resync;
 static unsigned infoSize, nameSize, aliasSize;
 static char *infoBase, *nameBase, *aliasBase;
-const char *fontdir = FONTDIR;
-const char *cachedir = CACHEDIR;
-const char *pidfilename = PIDFILE;
+static const char *fontdir = FONTDIR;
+static const char *cachedir = CACHEDIR;
+static const char *pidfilename = PIDFILE;
 
-int defaultres = 0;
-const int default_port = 7101;
+static int defaultres = 0;
+static const int default_port = 7101;
 
 struct fs_conn {
 	bool listen_unix;
@@ -126,14 +125,14 @@ struct fs_client {
 // Forward declarations
 static int fs_client_error(fs_client &client, int error);
 
-uid_t newuid = (uid_t)(-2);
-gid_t newgid = (gid_t)(-2);
+static uid_t newuid = (uid_t)(-2);
+static gid_t newgid = (gid_t)(-2);
 
-const char *sockname;
-const char *sockdir = "/tmp/.font-unix";
+static const char *sockname;
+static const char *sockdir = "/tmp/.font-unix";
 
 #define MAXENC 16	/* Maximum number of encodings */
-Encoding *encodings[MAXENC];
+static Encoding *encodings[MAXENC];
 
 static void
 version()
@@ -188,6 +187,8 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 		info(_("Sync in directory \"%s/%s\".\n"), fontdir, ttdir);
 
 	DIR *dirp = opendir(".");
+	if (dirp == NULL)
+		return 0;
 
 	while (dirent *de = readdir(dirp)) {
 		int namelen = strlen(de->d_name);
@@ -264,8 +265,7 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 		delete ttFont;
 		++nfonts;
 	}
-	if (dirp)
-		closedir(dirp);
+	closedir(dirp);
 
 	return nfonts;
 }
@@ -308,7 +308,6 @@ ttSyncAll(bool gslist = false)
 		error(_("directory \"%s\" does not exist!\n"), fontdir);
 		return -1;
 	}
-	DIR *dirp = opendir(".");
 
 	char *ttinfofilename = cachefile(TTINFO_LEAF);
 	if (!ttinfofilename)
@@ -324,7 +323,11 @@ ttSyncAll(bool gslist = false)
 	delete ttinfofilename;
 	delete ttnamefilename;
 
-	if (infoFile <= 0 || nameFile <= 0) {
+	if (infoFile == NULL || nameFile == NULL) {
+		if (infoFile)
+			fclose(infoFile);
+		if (nameFile)
+			fclose(nameFile);
 		error(_("cannot write to font database!\n"));
 		return -1;
 	}
@@ -339,12 +342,18 @@ ttSyncAll(bool gslist = false)
 	fwrite((void *)&info, 1, sizeof(info), nameFile);
 
 	int nfonts = ttSyncDir(infoFile, nameFile, ".", gslist);
+
+	DIR *dirp = opendir(".");
+	if (dirp == NULL)
+		return 0;
+
 	while (dirent *de = readdir(dirp)) {
 		chdir(fontdir);
 		if (de->d_name[0] != '.' && !chdir(de->d_name))
 			nfonts += ttSyncDir(infoFile, nameFile,
 					    de->d_name, gslist);
 	}
+	closedir(dirp);
 
 	fclose(infoFile);
 	fclose(nameFile);
@@ -523,8 +532,12 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 	debug("point %d, pixel %d, res %d\n",
 	      fp->point[0], fp->pixel[0], fp->resolution[0]);
 
-	if (!ttFont || ttFont->badFont())
+	if (!ttFont)
 		return 0;
+	if (ttFont->badFont()) {
+		delete ttFont;
+		return 0;
+	}
 
 	XFSFont *xfs = findFont(0);
 	if (!xfs) {
@@ -586,7 +599,8 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 	while (!(xfs->fe.buffer = (uint8_t *)allocMem(xfs->fe.buflen)))
 		if ((xfs->fe.buflen >>= 1) < MINFONTBUFSIZE) {
 			error(_("entering memory starved mode.\n"));
-			xfs->fid = 0;
+			xfs->fid = 0;	// Mark font slot as not used.
+			delete ttFont;
 			return 0;
 		}
 
@@ -594,13 +608,16 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 
 	int used = (xfs->fe.bitmaps + xfs->fe.bmplen) - xfs->fe.buffer;
 	int bmpoff = xfs->fe.bitmaps - xfs->fe.buffer;
-	xfs->fe.buffer = (uint8_t *)shrinkMem(xfs->fe.buffer, used);
-	if (xfs->fe.buffer) {
+	uint8_t *newbuf = (uint8_t *)shrinkMem(xfs->fe.buffer, used);
+
+	if (newbuf) {
+		xfs->fe.buffer = newbuf;
 		xfs->fe.buflen = used;
 		xfs->fe.bitmaps = xfs->fe.buffer + bmpoff;
 	} else {
-		xfs->fid = 0;	// XXX
+		xfs->fid = 0;	// Mark font slot as not used.
 		xfs = 0;
+		delete ttFont;
 	}
 
 	return xfs;
@@ -782,7 +799,7 @@ openXLFD(Rasterizer *raster, char *xlfdName, FontParams *fp, int fid)
 }
 
 // returns > 0 if ok
-int
+static int
 openTTFdb()
 {
 	infoSize = nameSize = aliasSize = 0;
@@ -797,7 +814,7 @@ openTTFdb()
 		return 0;
 
 	int fd = open(ttinfofilename, O_RDONLY);
-	if (0 >= fd) {
+	if (fd < 0) {
 		error(_("cannot open font database!\n"));
 		delete ttinfofilename;
 		return 0;
@@ -831,7 +848,7 @@ openTTFdb()
 		return -1;
 
 	fd = open(ttnamefilename, O_RDONLY);
-	if (0 >= fd) {
+	if (fd < 0) {
 		error(_("cannot open font database!\n"));
 		delete ttnamefilename;
 		return 0;
@@ -862,7 +879,7 @@ openTTFdb()
 	if (!stat("fonts.alias", &statbuf)) {
 		aliasSize = statbuf.st_size;
 		int fd = open("fonts.alias", O_RDONLY);
-		if (fd <= 0)
+		if (fd < 0)
 			return 0;
 		aliasBase = (char *)mmap(0L, aliasSize, PROT_READ, MAP_SHARED,
 					 fd, 0L);
@@ -877,7 +894,7 @@ openTTFdb()
 	return 1;
 }
 
-void
+static void
 closeTTFdb()
 {
 	if (infoSize)
@@ -962,9 +979,10 @@ fs_connection_setup(fs_conn &conn)
 		s_unix.sun_family = AF_UNIX;
 		sprintf(s_unix.sun_path, "%s/fs%d", sockdir, conn.port);
 		sockname = s_unix.sun_path;
+
+		old_umask = umask(0);
 		mkdir(sockdir, 01777);
 		unlink(s_unix.sun_path);
-		old_umask = umask(0);
 
 		if (bind(sd, (struct sockaddr *)&s_unix, sizeof(s_unix)) < 0) {
 			error(_("could not write to %s/, please check "
@@ -1215,7 +1233,8 @@ fs_client_error(fs_client &client, int error)
 	reply.sequenceNumber = client.seqno;
 	reply.length = sizeof(reply) >> 2;
 	reply.timestamp = 0;
-	reply.major_opcode = 0;
+	reply.major_opcode = client.req.reqType;
+	// No extensions supported, so set minor always to 0.
 	reply.minor_opcode = 0;
 
 	return write(client.sd, (void *)&reply, sizeof(reply));
@@ -1899,18 +1918,7 @@ fs_working(fs_client &client, Rasterizer *raster)
 
 		default:
 			debug("Unknown FS request 0x%02X !\n", client.req.reqType);
-			{
-			fsRequestError reply;
-			reply.type = FS_Error;
-			reply.request = FSBadRequest;
-			reply.sequenceNumber = client.seqno;
-			reply.length = sizeof(reply) >> 2;
-			reply.timestamp = 0;
-			reply.major_opcode = client.req.reqType;
-			reply.minor_opcode = client.req.data;
-
-			write(client.sd, (void *)&reply, sizeof(reply));
-			}
+			fs_client_error(client, FSBadRequest);
 			break;
 		}
 		debug("done.\n");
@@ -1923,7 +1931,7 @@ fs_working(fs_client &client, Rasterizer *raster)
  * pid file is properly disposed of when we are killed
  * possibly a better (more robust) signal handler could be written - sjc
  */
-void
+static void
 delPIDfile(int signal XFSTT_ATTR_UNUSED)
 {
 	unlink(pidfilename);
@@ -1952,7 +1960,7 @@ signal_setup(int sig_num, void (*sig_handler)(int))
 	sigaction(sig_num, &sig, NULL);
 }
 
-void
+static void
 setuidgid(char *name)
 {
 	struct passwd *pwent;
@@ -2035,10 +2043,10 @@ main(int argc, char **argv)
 			version();
 			return 0;
 		} else if (!strcmp(argv[i], "--inetd")) { // thanks Feanor
-			inetdConnection = 1;
-			multiConnection = 0;
+			inetdConnection = true;
+			multiConnection = false;
 		} else if (!strcmp(argv[i], "--once")) {
-			multiConnection = 0;
+			multiConnection = false;
 		} else if (!strcmp(argv[i], "--unstrap")) {
 			maxLastChar = UNSTRAPLIMIT;
 			warning(_("[unstrapped] you must start X11 with "
@@ -2157,4 +2165,3 @@ main(int argc, char **argv)
 
 	return 0;
 }
-
